#include "Mainwindow.hxx"
#include "widgets/BasicBlockWidget.hxx"
#include "qt.hxx"

#include <iostream>
#include <sstream>

#include <QtGui>

Mainwindow::Mainwindow()
{
    openAction = new QAction(tr("&Open"), this);
    // saveAction = new QAction(tr("&Save"), this);
    exitAction = new QAction(tr("E&xit"), this);

    connect(openAction, SIGNAL(triggered()), this, SLOT(open()));
    // connect(saveAction, SIGNAL(triggered()), this, SLOT(save()));
    connect(exitAction, SIGNAL(triggered()), qApp, SLOT(quit()));

    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(openAction);
    // fileMenu->addAction(saveAction);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAction);

    listWidget = new QListWidget();
    stackedWidget = new QStackedWidget();
    dockWidget = new QDockWidget(tr("Functions"), this);
    dockWidget->setAllowedAreas(Qt::LeftDockWidgetArea |
                                Qt::RightDockWidgetArea);
    dockWidget->setWidget(listWidget);
    addDockWidget(Qt::LeftDockWidgetArea, dockWidget);
    setCentralWidget(stackedWidget);

    connect(listWidget, SIGNAL(currentRowChanged(int)),
            stackedWidget, SLOT(setCurrentIndex(int)));

    setWindowTitle(tr("Notepad"));
}

void Mainwindow::quit()
{
    QMessageBox messageBox;
    messageBox.setWindowTitle(tr("Notepad"));
    messageBox.setText(tr("Do you really want to quit?"));
    messageBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    messageBox.setDefaultButton(QMessageBox::No);
    if (messageBox.exec() == QMessageBox::Yes)
        qApp->quit();
}

void Mainwindow::open() {
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"), "",
                                                    tr("Binaries (*)"));

    if (fileName != "") {
        curBin = new qtlldb::Binary(fileName.toStdString());

        std::vector<std::string> symbols = curBin->getSymbols();
        if (0 == symbols.size())
            populateSymbolInformation(".text");
        for (std::vector<std::string>::iterator it = symbols.begin();
             it != symbols.end();
             ++it) {
            populateSymbolInformation(*it);
        }
    }
}

void Mainwindow::populateSymbolInformation(const std::string& sym) {
    QTabWidget * w = new QTabWidget();

    // Listing
    QTableWidget * t = new QTableWidget();
    t->setColumnCount(3);
    t->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
    curBin->for_each_instruction(sym, [&t](long add, std::string bytes, std::string mnemonic) {
            int row = t->rowCount();
            std::stringstream s;
            t->setRowCount(t->rowCount() + 1);
            s << std::hex << add;
            t->setItem(row,0,new QTableWidgetItem(s.str().c_str()));
            s.str("");
            s << std::hex;
            for_each(bytes.begin(), bytes.end(), [&s](char c){s << (unsigned int)((unsigned char)c) << ' ';});
            t->setItem(row,1,new QTableWidgetItem(s.str().c_str()));
            t->setItem(row,2,new QTableWidgetItem(mnemonic.c_str() + 1));
        });
    w->addTab(t, "Listing");

    // CFG
    QGraphicsScene * scene = new QGraphicsScene;

    BasicBlockWidget * s1 = new BasicBlockWidget;
    scene->addItem(s1);
    s1->setFlag(QGraphicsItem::ItemIsMovable, true);

    BasicBlockWidget * s2 = new BasicBlockWidget;
    scene->addItem(s2);
    s2->setFlag(QGraphicsItem::ItemIsMovable, true);
    s2->moveBy(-200, 350);

    BasicBlockWidget * s3 = new BasicBlockWidget;
    scene->addItem(s3);
    s3->setFlag(QGraphicsItem::ItemIsMovable, true);
    s3->moveBy(100, 350);

    BasicBlockWidget * s4 = new BasicBlockWidget;
    scene->addItem(s4);
    s4->setFlag(QGraphicsItem::ItemIsMovable, true);
    s4->moveBy(400, 350);


    QGraphicsView * view = new QGraphicsView(scene);
    w->addTab(view, "CFG");

    listWidget->addItem(sym.c_str());
    stackedWidget->addWidget(w);
}
